%% Burcin Kisacikoglu
%% Contact : bkisaci1@jhu.edu
%% Johns Hopkins University


% The code below has two tasks: Calculate and save the forecasts. First it
% calls the necessary datafiles one by one and uses them in calculating the
% forecasts from the DSGE model via Dynare and BVAR. To be able to save the
% forecasts, first we initialize empty vector of bvar_forecast which we
% will fill it with the forecasts coming from BVAR.

for k=1:64
% for k = 1:40, this is for GB 1992-1996 coverage.
% for k = 1:153, this is for Blue Chip data sets 1997-2009 coverage.
% for k = 1:60, this is for Blue Chip data sets 1992-1996 coverage.

% The first step of the code is to make sure we are calling the files one
% by one. So the first part of the code takes the files which are
% sequentially named as datafile1,...datafile64. They are generated from 
% the real time data set. So say, datafile1 corresponds to January 1997 
% real time data.
% These datafiles include the training sample of 40 observations(1956Q1-1965Q4) 
% which are used in Smets-Wouters(2007). Training sample is available in the .zip file.
% Thus there are 64 of those. The estimation 
% starts from observation 42, 1966Q1. 
% We add the training sample to the datafiles because we want the BVAR code and
% Dynare to start from the same observation while doing the calculations.
% It is explained in detail in the bvar_ml_forecast.m below.


load(sprintf('datafile%d.mat',k));

% Remove the comments one by one to get the desired forecasts for the
% relevant coverage.
% load(sprintf('extdatafile%d.mat',k)); % This calls the datasets from
% 1992 to 1996. 
% load(sprintf('bcdadtafile%d.mat',k)); % This calls the Blue Chip
% datasets from 1997 to 2009.
% load(sprintf('extbcdatafile.mat',k)); % This calls the Blue Chip datasets
% from 1992 to 1996.


% The line below enables Dynare to use the called datafile by saving the observed
% variables in the datafile into another .mat file, model_data. Then we feed
% model_data to estimation options in Dynare.


save model_data dc dinve dy labobs pinfobs dw robs; 

% As default, Dynare clears the workspace before starting the computations. 
% This is undesriable since the index 'k' is also cleared from the workspace. 
% Hence we use option 'noclearall' to make sure that index 'k' is in the
% workspace. Here the MCMC algorithm is done for 100000 replications
% discarging 20000 of them. Then we compute 8 quarter ahead forecasts from the DSGE
% model.

dynare usmodel_v4 noclearall; 

% Next step is to recursively save the forecasts created by the DSGE model.
% To do that for efficiency we vectorize the forecasts. To be able to
% keep track of the process of the code we also generate forecast files 
% recursively, forecasts1, forecasts2,..., forecasts64. They are not saved
% into the hard disk, but it is easier to keep track of the process. 

eval(['forecasts' num2str(k) ' =oo_' ]); 
forecast(:,k)=oo_;

% Here oo_ is the .mat file that is generated by Dynare that has the estimation 
% and forecast outcomes such as forecasts, estimation results, mode files etc.
% generated by Dynare. 

save forecast;

% Next we load the mode file of the estimation, where the name for the
% .mod file is usmodel_v4. I had to modify the .mod file provided by Smets
% and Wouters to be able to do the estimations in Dynare 4. So load the
% mode file and save the modes and the hessian matrix generated from
% Bayesian estimation. 



load usmodel_v4_mode;
modefile(:,k)=xparam1;
save modefile;
hessian(:,:,k) = hh;
save hessian;



% After DSGE model forecasts, we have to do the same
% exercise for the Bayesian VAR. The BVAR code is the slightly modified version 
% of the one that is used in Smets and Wouters(2007) which is available in
% the AER webpage. 

% Before we do the BVAR estimations, we have to append the training sample
% to the datafiles that are going to be loaded to undertake the
% estimations. appending.m below appends the training sample to the datafiles. 
% To append the training sample, we have to replace the common observations between
% the training sample and the datafile. Since we have 8 quarters in common
% between them, we have to replace these 8 quarters with the observations 
% from the training sample. Thus before we append, we delete these common
% observations, then append the training sample, which ultimately replaces
% the common observations with the ones in the training sample.

% For BC files, do not use appending.m because the datafiles are already
% appended with the training sample. 

appending;

% Next we run the BVAR estimates to calculate ML and the forecasts. Please
% refer to bvar_ml_forecast.m for detailed annotation.

bvar_ml_forecast;

% So again we vectorize the BVAR forecasts to save them.
yhatfcast = yhatmat(:,:,2);
bvar_forecast(:,:,k) = yhatfcast ;
% Then we save the workspace all together. 
save bvar_forecast;

% Finally we generate bvar forecasting files to help us to keep track of
% the process.

eval(['bvar_fcast' num2str(k) '=yhatmat(:,:,2)']);


end 